package admin

import (
	"context"
	"gd_admin/apis"
	"gd_admin/errors"
	"gd_admin/impl/rbac"
	"encoding/base64"
	"time"

	"gd_admin/common.in/config"
	"github.com/astaxie/beego/orm"
)

func adminLoginById(req *apis.AdminLoginReq, reply *apis.AdminLoginReply) error {

	o := orm.NewOrm()
	adminName := ""
	err := o.Raw("select id, admin_name, admin_level from t_gd_admin where id = ?", req.Uid).QueryRow(&reply.AdminId, &adminName, &reply.Level)
	if err != nil {
		if err == orm.ErrNoRows {
			return errors.AdminNotExist
		}
		return errors.DataBaseError
	}
	var loginLog AdminLoginLog
	loginLog.AdminId = reply.AdminId
	loginLog.AdminName = adminName
	var timeLayout = "2006-01-02 15:04:05"
	timeNow := time.Now().Format(timeLayout)
	loginLog.LoginTime = timeNow
	o.Insert(&loginLog)

	// 获取用户权限信息
	reply.Access, reply.Resource, err = rbac.GetUserAccess(reply.AdminId)
	if err != nil {
		return err
	}

	return nil
}

func AdminLogin(ctx context.Context, req *apis.AdminLoginReq, reply *apis.AdminLoginReply) error {
	if (req.AdminPasswd == "" || req.AdminName == "") && req.Uid == 0 {
		return errors.ArgsError
	}
	if req.Uid > 0 {
		return adminLoginById(req, reply)
	}
	cryPasswd, err := config.AesEncrypt(req.AdminPasswd, CRYPTO_KEY)
	if err != nil {
		return errors.ArgsError
	}

	o := orm.NewOrm()
	originPass := ""
	var status int
	err = o.Raw("select id,admin_level,status, admin_password from t_gd_admin where admin_name = ?", req.AdminName).QueryRow(&reply.AdminId, &reply.Level, &status, &originPass)
	if err != nil {
		if err == orm.ErrNoRows {
			return errors.AdminNotExist
		}
		return errors.DataBaseError
	}

	if status == 1 {
		return errors.AdminNotExist
	}

	if originPass != base64.StdEncoding.EncodeToString(cryPasswd) {
		return errors.PasswordError
	}

	var loginLog AdminLoginLog
	loginLog.AdminId = reply.AdminId
	loginLog.AdminName = req.AdminName
	var timeLayout = "2006-01-02 15:04:05"
	timeNow := time.Now().Format(timeLayout)
	loginLog.LoginTime = timeNow
	o.Insert(&loginLog)

	// 获取用户权限信息
	reply.Access, reply.Resource, err = rbac.GetUserAccess(reply.AdminId)
	if err != nil {
		return err
	}

	return nil
}
